* ------------------------------------------------------------ *
*  Program...........: XUSE.PRG
*  Author............: Greg Dunn
*  Project...........: Reusable Application Foundation
*  Created...........: 06/25/1993
*  Copyright.........: (c) TRW, 1993
*  Description.......:
*  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - *
* 		Employ as an alternative to the USE command.
* 
* 		Called to select or (as needed) open a table.
* 
* 		If the table is already open and the tcMode parameter
* 		is empty, XUSE simply selects the work area of the table.
* 
* 		If the table is not already open, XUSE finds an available
* 		work area and opens the table in the mode specified (if any).
* 
* 		If table is open, but XUSE has been called with a non-empty
* 		mode parameter, XUSE selects the table, closes it, and reopens
* 		it in the same area in the mode specified.  In other words,
* 		passing the tcMode parameter causes XUSE definitively to make
* 		sure the table is open in the specified mode.
* 
* 		If the use is successful, XUSE returns .T.  Otherwise it
* 		issues an error message and returns .F.
* 
*  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - *
*  Calling Samples...:
*  Parameter List....:
*		tcTable	- Table to select or open
*		tcMode	- Opening mode ("EXCLUSIVE", "ALIAS POOP", etc.)
*		tcOrder	- Index order
*  Major change list.:
*		6/25/93 GTD - Added third argument, tcOrder, allowing
*		specification and setting of an ORDER tag without
*		re-opening an open table.  Previously, the ORDER had
*		to be passed as part of the tcMode, which would cause
*		the table to be re-opened.
* ------------------------------------------------------------ *

PARAMETERS tcTable, tcMode, tcOrder
PRIVATE tcTable, tcMode, tcOrder, lnParms, llSuccess, lcTab1
PRIVATE lcErrorWas

* Check parameters
lnParms = PARAMETERS()
IF lnParms < 1 OR lnParms > 3
	WAIT WINDOW "XUSE:  Invalid parameter count."
	RETURN .F.
ENDIF

tcMode  = IIF(lnParms = 1, "", ALLTRIM(tcMode))
tcOrder = IIF(lnParms = 1 OR lnParms = 2, "", tcOrder)

* Look for open database
lcTab1 = ALLTRIM(tcTable)
IF RAT("\", lcTab1) > 0
	lcTab1 = SUBSTR(lcTab1, RAT("\", lcTab1)+1)
ENDIF
IF RAT("/", lcTab1) > 0
	lcTab1 = SUBSTR(lcTab1, RAT("/", lcTab1)+1)
ENDIF
IF RAT(".", lcTab1)>0
	lcTab1 = LEFT(lcTab1, RAT(".", lcTab1)-1)
ENDIF
tcTable = ALLTRIM(tcTable)

llSuccess = .T.
IF USED(lcTab1) AND EMPTY(tcMode)
	SELE (lcTab1)
	IF !EMPTY(tcOrder)
		SET ORDER TO &tcOrder
	ENDIF
ELSE
	IF USED(lcTab1)
		* We need to re-open with the specified mode, but want
		* to do so in the same work area...
		SELE (lcTab1)
	ELSE
		SELE 0
	ENDIF

	lcErrorWas = ON("error")
	ON ERROR DO XUseErr WITH PROGRAM(), ERROR(), MESSAGE()
	USE (LOCFILE(tcTable,"DBF","Where is " + tcTable + "?")) &tcMode
	IF ERROR() = 0 AND !EMPTY(tcOrder)
		SET ORDER TO &tcOrder
	ENDIF
	ON ERROR &lcErrorWas

ENDIF

RETURN llSuccess


* ------------------------------------------------------------ *
PROCEDURE XUseErr
* ------------------------------------------------------------ *
* Error handler for XUSE()
* ------------------------------------------------------------ *
PARAMETERS lcErrorPrg, lnErrorNum, lcErrorMsg
PRIVATE lcErrorPrg, lnErrorNum, lcErrorMsg
* called by ON ERROR when the use fails in XUSE.

ON ERROR &lcErrorWas
llSuccess = .F.
WAIT WINDOW "XUSE:  Error # "+ALLTRIM(STR(lnErrorNum))+ ;
	", "+ALLTRIM(lcErrorMsg) + " (" + UPPER(tcTable) + ")" ;
	TIMEOUT 2
RETURN .F.
